@@ -1,4 +1,4 @@
-boot.relimp.formula <- function(formula, data, na.action, ..., subset=NULL){
+boot.relimp.formula <- function(formula, data, weights, na.action, ..., subset=NULL){
     if (missing(formula)) stop("formula missing")
     if (missing(na.action)) 
         na.action <- getOption("na.action")
@@ -12,10 +12,9 @@ boot.relimp.formula <- function(formula, data, na.action, ..., subset=NULL){
     terms <- attr(mf,"terms")
     resp <- attr(terms,"response")
     if (resp != 1 ) stop("incorrect formula") 
-    if (max(attr(terms,"order")) != 1) stop ("formula contains higher order terms")
+    if (max(attr(terms,"order")) > 2) stop ("formula contains terms of order higher than 2")
     if (attr(terms,"intercept") != 1) stop ("model must contain intercept")
 
-    if (any(as.logical(lapply(mf,is.factor)))) stop("The model must not contain any factors!")
     if (!is.null(attr(mf,"na.action"))) warning(naprint(attr(mf,"na.action")))
     if (!is.null(dim(model.response(mf)))){
           if (ncol(model.response(mf))>1) stop("too many response variables")
@@ -24,8 +23,9 @@ boot.relimp.formula <- function(formula, data, na.action, ..., subset=NULL){
     ## selection of columns from model based on formula below 
     ##               works even in case of multi-column terms such as poly(x2,3)
 
-    DATA <- as.matrix(mf[,c(resp,which(rowSums(attr(terms,"factors"))>0))])
+    #DATA <- as.matrix(mf[,c(resp,which(rowSums(attr(terms,"factors"))>0))])
+    #weights <- mf$"(weights)"
     
-    bt <- do.call("boot.relimp", list(DATA, ...))
+    bt <- do.call("boot.relimp", list(lm(mf), ...))
     bt
 }
